<?php
class tx_immoscout2ttnews_import extends tx_scheduler_Task {

	private $extConf;
	private $oImmocaster;

	function execute() {
		$this->extConf = unserialize($GLOBALS['TYPO3_CONF_VARS']['EXT']['extConf']['immoscout2ttnews']);

		// load SDK
		require_once(t3lib_extMgm::extPath('immoscout2ttnews') . 'Immocaster/Sdk.php');

		$this->oImmocaster = Immocaster_Sdk::getInstance(
			'is24',
			$this->extConf['basic.']['key'],
			$this->extConf['basic.']['secret']
		);

		// use cUrl
		if($GLOBALS['TYPO3_CONF_VARS']['SYS']['curlUse'] == 1) {
			$this->oImmocaster->setReadingType('curl');
		}

		// work on live
		if($this->extConf['basic.']['live']) {
			$this->oImmocaster->setRequestUrl('live');
		}

		// get data as JSON (instead XML)
		$this->oImmocaster->setContentResultType('json');

		// connect to DB
		$aDatabase = array(
			'mysql',
			TYPO3_db_host,
			TYPO3_db_username,
			TYPO3_db_password,
			TYPO3_db
		);
		$this->oImmocaster->setDataStorage($aDatabase);

		// application not authenticated
		if(!$this->oImmocaster->oDataStorage->getApplicationToken()) {
			t3lib_div::devLog('[tx_immoscout2ttnews_import]: Application is not authenticated', 'scheduler', 2);
			throw new t3lib_exception('Application is not authenticated');
		}

		/* Merkzettel
		if(is_array($res->{'shortlist.shortlists'})) {
			foreach($res->{'shortlist.shortlists'} as $shortlistId => $shortlist) {
				$res2 = $this->oImmocaster->getShortlistEntry(array('username'=>'me', 'entry'=>$shortlistId));
				$res2 = json_decode($res);
				var_dump($res2);exit;
			}
		}*/

		#$res = $this->oImmocaster->getShortlist(array('username'=>'me'));
		// Clean database
		#$GLOBALS['TYPO3_DB']->exec_DELETEquery($this->importTable, '_realestatetype IN ("' . $this->realestatetype . '") && _geocode IN ("' . $this->geocodes . '")');
		// Mark records as deleted
		$GLOBALS['TYPO3_DB']->exec_UPDATEquery('tt_news', '_immoscout_realestatetype IN ("' . $this->realestatetype . '") && _immoscout_geocode IN ("' . $this->geocodes . '")', array('deleted' => 1));

		// Request data from immobilienscout24
		foreach(t3lib_div::trimExplode(',', $this->realestatetype) as $realestatetype) {
			foreach(t3lib_div::trimExplode(',', $this->geocodes) as $geocode) {
				// get data from immoscout
				$res = $this->oImmocaster->channelSearch(array(
					'channel' => 'hp',
					'username' => 'me',
					'realestatetype' => $realestatetype,
					'geocodes' => $geocode
				));
				#$res = $this->oImmocaster->regionSearch(array('realestatetype'=>'apartmentrent','geocodes'=>1));
				$res = json_decode($res);

				// something went wrong...
				if(is_array($res->{'common.messages'})) {
					foreach($res->{'common.messages'} as $msg) {
						$errorMsg = 'Code: ' . $msg->message->messageCode . ', message: ' . $msg->message->message;
						t3lib_div::devLog('[tx_immoscout2ttnews_import]: ' . $errorMsg, 'scheduler', 2);
						throw new t3lib_exception($errorMsg);
					}
				}

				// store data in database
				$entries = $res->{'resultlist.resultlist'}->resultlistEntries[0]->resultlistEntry;
				if(is_object($entries)) {
					$this->saveToDB($entries, $realestatetype, $geocode);
				} elseif(is_array($entries)) {
					foreach($entries as $entry) {
						$this->saveToDB($entry, $realestatetype, $geocode);
					}
				}
			}
		}
		// clear cache
		if($this->extConf['basic.']['clear_cache']) {
			$list_cache = t3lib_div::trimExplode(',', $this->extConf['basic.']['clear_cache']);
			if(TYPO3_UseCachingFramework) {
				$pageCache = $GLOBALS['typo3CacheManager']->getCache('cache_pages');
				$pageSectionCache = $GLOBALS['typo3CacheManager']->getCache('cache_pagesection');

				$pageIds = $GLOBALS['TYPO3_DB']->cleanIntArray($list_cache);
				foreach($pageIds as $pageId) {
					$pageCache->flushByTag('pageId_' . $pageId);
					$pageSectionCache->flushByTag('pageId_' . $pageId);
				}
			} else {
				$GLOBALS['TYPO3_DB']->exec_DELETEquery('cache_pages', 'page_id IN (' . implode(',', $GLOBALS['TYPO3_DB']->cleanIntArray($list_cache)) . ')');
				$GLOBALS['TYPO3_DB']->exec_DELETEquery('cache_pagesection', 'page_id IN (' . implode(',', $GLOBALS['TYPO3_DB']->cleanIntArray($list_cache)) . ')');
			}
		}
		return true;
	}

	/**
	 * Insert data of real estate into database
	 *
	 * @param object $data
	 * @param string $realestatetype: Type of real estate
	 * @param integer $geocode: GEO code
	 */
	private function saveToDB($data, $realestatetype, $geocode) {
		// Get more data
		$exposeData = $this->oImmocaster->getExpose(array('exposeid' => $data->realEstateId));
		$exposeData = json_decode($exposeData);
		$exposeData = $exposeData->{'expose.expose'};

		// Delete old record
		#$GLOBALS['TYPO3_DB']->exec_DELETEquery($this->importTable, 'realEstateId = ' . $data->realEstateId);

		$realestateData = $data->{'resultlist.realEstate'};

		// Get images
		$exposeAttachments = $this->oImmocaster->getAttachment(array('exposeid' => $data->realEstateId));
		$exposeAttachments = json_decode($exposeAttachments);
		$exposeAttachments = $exposeAttachments->{'common.attachments'}[0]->attachment;
		$ttNewsImages = array();
		$ttNewsImageCaptions = array();
		if(is_object($exposeAttachments)) {
			// Get image no. 3 (best quality)
			$file = $this->saveImage($exposeAttachments->urls[0]->url[3]->{'@href'}, $data->realEstateId);
			if($file) {
				$ttNewsImages[] = $file;
				$ttNewsImageCaptions[] = $exposeAttachments->title;
			}
		} elseif(is_array($exposeAttachments)) {
			foreach($exposeAttachments as $exposeAttachment) {
				// Get image no. 3 (best quality)
				$file = $this->saveImage($exposeAttachment->urls[0]->url[3]->{'@href'}, $data->realEstateId);
				if($file) {
					$ttNewsImages[] = $file;
					$ttNewsImageCaptions[] = $exposeAttachment->title;
				}
			}
		}

		$ttNews = array(
			'deleted'							=> 0,
			'crdate'							=> strtotime($data->{'@creation'}),
			'cruser_id'							=> $GLOBALS['BE_USER']->user['uid'],
			'tstamp'							=> strtotime($data->{'@modification'}),
			'pid'								=> $this->importPid,
			'title'								=> $realestateData->title,
			'datetime'							=> strtotime($data->{'@creation'}),
			'short'								=> $exposeData->realEstate->descriptionNote,
			'image'								=> implode(',', $ttNewsImages),
			'imagecaption'						=> rtrim(implode(PHP_EOL, $ttNewsImageCaptions), ','),
			'imagetitletext'					=> rtrim(implode(PHP_EOL, $ttNewsImageCaptions), ','),
			'author'							=> $exposeData->contactDetails->firstname . ' ' . $exposeData->contactDetails->lastname,
			'ext_url'							=> 'http://www.immobilienscout24.de/expose/' . $data->realEstateId,
			'tx_jfeasymaps_street'				=> $realestateData->address->street . ' ' . $realestateData->address->houseNumber,
			'tx_jfeasymaps_city'				=> $realestateData->address->postcode . ' ' . $realestateData->address->city,
			'tx_jfeasymaps_land'				=> $exposeData->realEstate->address->geoHierarchy->country->name,
			'tx_jfeasymaps_infofenster'			=> $realestateData->title,
			'immoscout_data'					=> json_encode($exposeData),
			'immoscout_exposeid'				=> $data->realEstateId,
			'_immoscout_realestatetype'			=> $realestatetype,
			'_immoscout_geocode'				=> $geocode,
		);

		// Check if old record is available
		$found = $GLOBALS['TYPO3_DB']->exec_SELECTgetSingleRow(
			'uid',
			'tt_news',
			'immoscout_exposeid = ' . $data->realEstateId
		);

		// Update record
		if($found) {
			$GLOBALS['TYPO3_DB']->exec_UPDATEquery(
				'tt_news',
				'immoscout_exposeid = ' . $data->realEstateId,
				$ttNews
			);
		// Insert record
		} else {
			$GLOBALS['TYPO3_DB']->exec_INSERTquery(
				'tt_news',
				$ttNews
			);
		}

		/*$dbArray = array(
			'crdate'							=> strtotime($data->{'@creation'}),
			'cruser_id'							=> $GLOBALS['BE_USER']->user['uid'],
			'tstamp'							=> strtotime($data->{'@modification'}),
			'pid'								=> $this->importPid,
			'realEstateId'						=> $data->realEstateId,
			'externalId'						=> $exposeData->realEstate->externalId,
			'title'								=> $realestateData->title,
			'address_street'					=> $realestateData->address->street,
			'address_houseNumber'				=> $realestateData->address->houseNumber,
			'address_postcode'					=> $realestateData->address->postcode,
			'address_city'						=> $realestateData->address->city,
			'address_quarter'					=> $realestateData->address->quarter,
			'address_latitude'					=> $realestateData->address->wgs84Coordinate->latitude,
			'address_longitude'					=> $realestateData->address->wgs84Coordinate->longitude,
			'address_preciseHouseNumber'		=> $realestateData->address->preciseHouseNumber ? 1 : 0,
			'address_geoHierarchy_continent_geoCodeId'		=> $exposeData->realEstate->address->geoHierarchy->continent->geoCodeId,
			'address_geoHierarchy_continent_fullGeoCodeId'	=> $exposeData->realEstate->address->geoHierarchy->continent->fullGeoCodeId,
			'address_geoHierarchy_country_name'				=> $exposeData->realEstate->address->geoHierarchy->country->name,
			'address_geoHierarchy_country_geoCodeId'		=> $exposeData->realEstate->address->geoHierarchy->country->geoCodeId,
			'address_geoHierarchy_country_fullGeoCodeId'	=> $exposeData->realEstate->address->geoHierarchy->country->fullGeoCodeId,
			'address_geoHierarchy_region_name'				=> $exposeData->realEstate->address->geoHierarchy->region->name,
			'address_geoHierarchy_region_geoCodeId'			=> $exposeData->realEstate->address->geoHierarchy->region->geoCodeId,
			'address_geoHierarchy_region_fullGeoCodeId'		=> $exposeData->realEstate->address->geoHierarchy->region->fullGeoCodeId,
			'address_geoHierarchy_city_name'				=> $exposeData->realEstate->address->geoHierarchy->city->name,
			'address_geoHierarchy_city_geoCodeId'			=> $exposeData->realEstate->address->geoHierarchy->city->geoCodeId,
			'address_geoHierarchy_city_fullGeoCodeId'		=> $exposeData->realEstate->address->geoHierarchy->city->fullGeoCodeId,
			'address_geoHierarchy_quarter_name'				=> $exposeData->realEstate->address->geoHierarchy->quarter->name,
			'address_geoHierarchy_quarter_geoCodeId'		=> $exposeData->realEstate->address->geoHierarchy->quarter->geoCodeId,
			'address_geoHierarchy_quarter_fullGeoCodeId'	=> $exposeData->realEstate->address->geoHierarchy->quarter->fullGeoCodeId,
			'address_geoHierarchy_neighbourhood_geoCodeId'	=> $exposeData->realEstate->address->geoHierarchy->neighbourhood->geoCodeId,
			'price_value'						=> $realestateData->price->value,
			'price_currency'					=> $realestateData->price->currency,
			'price_marketingType'				=> $realestateData->price->marketingType,
			'price_priceIntervalType'			=> $realestateData->price->priceIntervalType,
			'price_additionalCosts_value'		=> $exposeData->realEstate->additionalCosts->value,
			'livingSpace'						=> $realestateData->livingSpace,
			'totalFloorSpace'					=> $exposeData->realEstate->totalFloorSpace,
			'netFloorSpace'						=> $exposeData->realEstate->netFloorSpace,
			'numberOfRooms'						=> $realestateData->numberOfRooms,
			'builtInKitchen'					=> $realestateData->builtInKitchen ? 1 : 0,
			'balcony'							=> $realestateData->balcony ? 1 : 0,
			'certificateOfEligibilityNeeded'	=> $realestateData->certificateOfEligibilityNeeded ? 1 : 0,
			'garden'							=> $realestateData->garden ? 1 : 0,
			'floorplan'							=> $realestateData->floorplan ? 1 : 0,
			'courtage_hasCourtage'				=> $realestateData->courtage->hasCourtage ? 1 : 0,
			'locationNote'						=> $exposeData->realEstate->locationNote,
			'descriptionNote'					=> $exposeData->realEstate->descriptionNote,
			'otherNote'							=> $exposeData->realEstate->otherNote,
			'apartmentType'						=> $exposeData->realEstate->apartmentType,
			'gastronomyType'					=> $exposeData->realEstate->gastronomyType,
			'listed'							=> $exposeData->realEstate->listed,
			'floor'								=> $exposeData->realEstate->floor,
			'lift'								=> $exposeData->realEstate->lift ? 1 : 0,
			'assistedLiving'					=> $exposeData->realEstate->assistedLiving ? 1 : 0,
			'terrace'							=> $exposeData->realEstate->terrace,
			'numberSeats'						=> $exposeData->realEstate->numberSeats,
			'cellar'							=> $exposeData->realEstate->cellar,
			'deposit'							=> $exposeData->realEstate->deposit,
			'commercializationType'				=> $exposeData->realEstate->commercializationType,
			'handicappedAccessible'				=> $exposeData->realEstate->handicappedAccessible,
			'condition'							=> $exposeData->realEstate->condition,
			'numberOfParkingSpaces'				=> $exposeData->realEstate->numberOfParkingSpaces,
			'interiorQuality'					=> $exposeData->realEstate->interiorQuality,
			'freeFrom'							=> $exposeData->realEstate->freeFrom,
			'buildingEnergyRatingType'			=> $exposeData->realEstate->buildingEnergyRatingType,
			'energyConsumptionContainsWarmWater'=> $exposeData->realEstate->energyConsumptionContainsWarmWater,
			'guestToilet'						=> $exposeData->realEstate->guestToilet,
			'price_baseRent'					=> $exposeData->realEstate->baseRent,
			'price_totalRent'					=> $exposeData->realEstate->totalRent,
			'price_serviceCharge'				=> $exposeData->realEstate->serviceCharge,
			'heatingCostsInServiceCharge'		=> $exposeData->realEstate->heatingCostsInServiceCharge,
			'petsAllowed'						=> $exposeData->realEstate->petsAllowed,
			'energyPerformanceCertificate'		=> $exposeData->realEstate->energyPerformanceCertificate ? 1 : 0,
			'contact_salutation'				=> $exposeData->contactDetails->salutation,
			'contact_firstname'					=> $exposeData->contactDetails->firstname,
			'contact_lastname'					=> $exposeData->contactDetails->lastname,
			'contact_phoneNumberCountryCode'	=> $exposeData->contactDetails->phoneNumberCountryCode,
			'contact_phoneNumberAreaCode'		=> $exposeData->contactDetails->phoneNumberAreaCode,
			'contact_phoneNumberSubscriber'		=> $exposeData->contactDetails->phoneNumberSubscriber,
			'contact_phoneNumber'				=> $exposeData->contactDetails->phoneNumber,
			'contact_faxNumberCountryCode'		=> $exposeData->contactDetails->faxNumberCountryCode,
			'contact_faxNumberAreaCode'			=> $exposeData->contactDetails->faxNumberAreaCode,
			'contact_faxNumberSubscriber'		=> $exposeData->contactDetails->faxNumberSubscriber,
			'contact_faxNumber'					=> $exposeData->contactDetails->faxNumber,
			'contact_cellPhoneNumberCountryCode'=> $exposeData->contactDetails->cellPhoneNumberCountryCode,
			'contact_cellPhoneNumberAreaCode'	=> $exposeData->contactDetails->cellPhoneNumberAreaCode,
			'contact_cellPhoneNumberSubscriber'	=> $exposeData->contactDetails->cellPhoneNumberSubscriber,
			'contact_cellPhoneNumber'			=> $exposeData->contactDetails->cellPhoneNumber,
			'contact_address_street'			=> $exposeData->contactDetails->address->street,
			'contact_address_houseNumber'		=> $exposeData->contactDetails->address->houseNumber,
			'contact_address_postcode'			=> $exposeData->contactDetails->address->postcode,
			'contact_address_city'				=> $exposeData->contactDetails->address->city,
			'contact_countryCode'				=> $exposeData->contactDetails->countryCode,
			'contact_company'					=> $exposeData->contactDetails->company,
			'contact_homepageUrl'				=> $exposeData->contactDetails->homepageUrl,
			'_realestatetype'					=> $realestatetype,
			'_geocode'							=> $geocode
		);

		// Check if old record is available
		$found = $GLOBALS['TYPO3_DB']->exec_SELECTgetSingleRow(
			'uid',
			$this->importTable,
			'realEstateId = ' . $data->realEstateId
		);

		// Update record
		if($found) {
			$GLOBALS['TYPO3_DB']->exec_UPDATEquery(
				$this->importTable,
				'realEstateId = ' . $data->realEstateId,
				$dbArray
			);
		// Insert record
		} else {
			$GLOBALS['TYPO3_DB']->exec_INSERTquery(
				$this->importTable,
				$dbArray
			);
		}*/
	}

	/**
	 * Save images of an expose in filesystem
	 *
	 * @param string $imagePath: URL of image (e.g. http://picture.immobilienscout24.de/files/basic001/N/114/848/667/114848667-1.jpg?3138973329)
	 * @param integer $id: Expose ID
	 * @return string: (new) filename of image
	 */
	private function saveImage($imagePath, $id) {
		$newFilename = array_shift(explode('?', basename($imagePath)));
		$copyTo = t3lib_div::getIndpEnv('TYPO3_DOCUMENT_ROOT') . '/uploads/pics/' . $newFilename;
		$success = copy($imagePath, $copyTo);
		return $success ? $newFilename : false;
	}
}